#ifndef IOCLASSES_H_INCLUDED
#define IOCLASSES_H_INCLUDED


#include "stm32f4xx_conf.h"
#include <cmath>

/*****C_ADC_DEFINES*********/
#define MAX_ARY 3
#define ADC_MASK 0x7FF



/*******************CLASS DigitalInput**********************/
class C_DI
{
private:
    bool aryFP[MAX_ARY];
    bool aryFN[MAX_ARY];
    bool value;
    GPIO_TypeDef *Port;
    uint16_t pinNum;

public:
    bool state(void);
    bool FP(int Num);
    bool FN(int Num);
    GPIO_TypeDef *getPort();
    uint16_t getPinNum();

    bool operator==(bool val);


    //Konstruktor
    C_DI(GPIO_TypeDef *Port, uint16_t pinNum);
    //Destruktor
    ~C_DI();

};


/*******************CLASS DigitalOutput**********************/
class C_DO
{
private:
    bool value;
    GPIO_TypeDef *Port;
    uint16_t pinNum;
    bool oldVal;

public:
    void on(void);
    void off(void);
    bool state(void);
    bool FP(void);
    bool FN(void);

    void operator=(bool val);
    bool operator==(bool val);

    //Konstruktor
    C_DO(GPIO_TypeDef *Port, uint16_t pinNum);
    //Destruktor
    ~C_DO();

};

/*******************CLASS AnalogInput**********************/
class C_ADC
{
private:

    uint16_t value;
    GPIO_TypeDef *Port;
    ADC_TypeDef* ADCx;
    uint16_t pinNum;
    uint16_t adcAry[0x800];
    int adcAryIndex;

    uint16_t median[3];
    uint16_t slAverage[3];

    uint16_t averNiveau;
    bool averValid;

    //Private-Methoden


public:

    bool averageValid(void);
    uint16_t averageValue(void);
    double COR(void);

    void updateValue(void);
    uint16_t getAryVal(void);
    void updateAryADC(void);
    void medianGlattung(void);
    void slidingAverage(void);
    uint16_t getValue(void);
    uint16_t getCorNiveau(int offset);
    uint16_t getAverageNiveau(void);

    void reset(void);


    //Konstruktor
    C_ADC(GPIO_TypeDef *Port, uint16_t pinNum, ADC_TypeDef* ADCx, uint8_t ADC_Channel);
    //Destruktor
    ~C_ADC();

};


#endif /* IOCLASSES_H_INCLUDED */
