/***********************************************************
* In/Output Pin Klassen
* Author:
* Datum: 05.02.2015
*
************************************************************/

#include "IOClasses.h"


/*******************CLASS DigitalInput**********************/

C_DI::C_DI(GPIO_TypeDef *Port, uint16_t pinNum)
{
        GPIO_InitTypeDef GPIO_InitStructure; //GPIO Strcut
        GPIO_StructInit(&GPIO_InitStructure);
        //Takte aktivieren
        if(Port == GPIOA) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
        else if(Port == GPIOB) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
        else if(Port == GPIOC) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
        else if(Port == GPIOD) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
        else if(Port == GPIOE) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);

        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
        GPIO_InitStructure.GPIO_Pin = pinNum;
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
        GPIO_Init(Port, &GPIO_InitStructure);

        this->value = false;
        this->Port = Port;
        this->pinNum = pinNum;
}


bool C_DI::state(void)
{
    return static_cast<bool>(GPIO_ReadInputDataBit(this->Port , this->pinNum));

}

bool C_DI::FN(int Num)
{
    //Maximale Groese ueberpruefen
    if(Num >= MAX_ARY) return false;

    bool actVal = this->state();
    bool retVal = false;

    if(actVal && !aryFP[Num])
    {
        retVal = true;
    }

    aryFP[Num] = actVal;
    return retVal;
}

bool C_DI::FP(int Num)
{
    //Maximale Groese ueberpruefen
    if(Num >= MAX_ARY) return false;

    bool actVal = this->state();
    bool retVal = false;

    if(actVal && !aryFN[Num])
    {
        retVal = true;
    }

    aryFN[Num] = actVal;
    return retVal;
}

GPIO_TypeDef *C_DI::getPort()
{
    return this->Port;
}

uint16_t C_DI::getPinNum()
{
    return this->pinNum;
}

bool C_DI::operator==(bool val)
{
    return (this->state() == val) ? true : false;
}

C_DI::~C_DI()
{
    ;
}
/***********END OF DIGITAL INPUT***********************/





/*******************CLASS DigitalOutput**********************/

C_DO::C_DO(GPIO_TypeDef *Port, uint16_t pinNum)
{
        GPIO_InitTypeDef GPIO_InitStructure; //GPIO Strcut
        GPIO_StructInit(&GPIO_InitStructure);

        //Takte aktivieren
        if(Port == GPIOA) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
        else if(Port == GPIOB) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
        else if(Port == GPIOC) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
        else if(Port == GPIOD) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
        else if(Port == GPIOE) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);

        GPIO_InitStructure.GPIO_Pin = pinNum;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
        GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
        GPIO_Init(Port, &GPIO_InitStructure);

        GPIO_ResetBits(Port, pinNum);

        this->value = false;
        this->Port = Port;
        this->pinNum = pinNum;
        this->off();
}

C_DO::~C_DO()
{
    ;
}

void C_DO::on(void)
{
    GPIO_SetBits(this->Port, this->pinNum);
    this->value = true;
}

void C_DO::off(void)
{
    GPIO_ResetBits(this->Port, this->pinNum);
    this->value = false;
}

bool C_DO::state(void)
{
    return static_cast<bool>(GPIO_ReadOutputDataBit(this->Port, this->pinNum));
}

bool C_DO::FP(void)
{
    bool Val = this->state();
    bool retVal = false;

    if(Val && !oldVal)
    {
        retVal = true;
    }

    oldVal = Val;
    return retVal;
}


bool C_DO::FN(void)
{
    bool Val = this->state();
    bool retVal = false;
    if(!Val && oldVal)
    {
        retVal = true;
    }

    oldVal = Val;
    return retVal;

}

void C_DO::operator=(bool val)
{
    (val) ? this->on() : this->off();
}

bool C_DO::operator==(bool val)
{
    return (this->state() == val) ? true : false;
}


/***********END OF DIGITAL OUTPUT***********************/



/*******************CLASS AnalogInput**********************/

C_ADC::C_ADC(GPIO_TypeDef *Port, uint16_t pinNum, ADC_TypeDef* ADCx, uint8_t ADC_Channel)
{


        this->Port = Port;
        this->pinNum = pinNum;
        this->ADCx = ADCx;

        this->value=0x1000; //12bit
        this->averValid = false;
        this->adcAryIndex=0;

        for(int i = 0 ; i < 0x800 ; ++i)
        {
            adcAry[i] = 0;
        }

        //ADC_CommonInitTypeDef ADC_CommonStruct;
        uint32_t ADCx_RCC_Chanel;
        ADC_InitTypeDef ADC_Init_structure; //Structure for adc confguration
        ADC_CommonInitTypeDef ADC_Common_structure;
        GPIO_InitTypeDef GPIO_InitStructure; //GPIO Strcut

        ADC_StructInit(&ADC_Init_structure);
        ADC_CommonStructInit(&ADC_Common_structure);
        GPIO_StructInit(&GPIO_InitStructure);


        ADC_Common_structure.ADC_TwoSamplingDelay = ADC_TwoSamplingDelay_5Cycles;
        ADC_Common_structure.ADC_Prescaler = ADC_Prescaler_Div2;
        ADC_Common_structure.ADC_Mode = ADC_Mode_Independent;
        ADC_CommonInit(&ADC_Common_structure);


        //Kanal fuer den ADC_Clock des entsprechenden Kanals definieren
        if(ADCx == ADC1) ADCx_RCC_Chanel = RCC_APB2Periph_ADC1;
        if(ADCx == ADC2) ADCx_RCC_Chanel = RCC_APB2Periph_ADC2;
        if(ADCx == ADC3) ADCx_RCC_Chanel = RCC_APB2Periph_ADC3;
        //Enable Clock
        RCC_APB2PeriphClockCmd(ADCx_RCC_Chanel,ENABLE);//The ADC is connected the APB2 peripheral bus
        RCC_AHB1PeriphClockCmd(RCC_AHB1ENR_GPIOCEN,ENABLE);//Clock for the ADC port C

        //Takte aktivieren
        if(Port == GPIOA) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
        else if(Port == GPIOB) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
        else if(Port == GPIOC) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
        else if(Port == GPIOD) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
        else if(Port == GPIOE) RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);

        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
        GPIO_InitStructure.GPIO_Pin = pinNum;
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_Init(Port, &GPIO_InitStructure);

        ADC_Init_structure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
        ADC_Init_structure.ADC_Resolution = ADC_Resolution_12b;
        ADC_Init_structure.ADC_ContinuousConvMode = ENABLE; //the conversion is continuous, the input data is converted more than once
        ADC_Init_structure.ADC_DataAlign = ADC_DataAlign_Right;//data converted will be shifted to right
        ADC_Init_structure.ADC_ExternalTrigConv = ENABLE; // conversion is synchronous with TIM1 and CC1 (actually I'm not sure about this one :/)
        ADC_Init_structure.ADC_NbrOfConversion = 1;//I think this one is clear :p
        ADC_Init_structure.ADC_ScanConvMode = DISABLE;//The scan is configured in one channel

        ADC_Init(ADCx,&ADC_Init_structure);//Initialize ADC with the previous configuration

        ADC_RegularChannelConfig(ADCx, ADC_Channel, 1, ADC_SampleTime_144Cycles);
        //Aktiviere ADC
        ADC_Cmd(ADCx, ENABLE);
        //Trigerre erstmalige Messung
        ADC_SoftwareStartConv(ADCx);

        //ADC-Kanal sichern

} //end of Konstruktor ADC




void C_ADC::updateValue()
{
    this->value = ADC_GetConversionValue(this->ADCx);
}

void C_ADC::updateAryADC()
{
    this->updateValue();
    adcAry[adcAryIndex++] = this->value;
    adcAryIndex &= ADC_MASK;
    return;
}

uint16_t C_ADC::getAryVal(void)
{
    return this->adcAry[(adcAryIndex-1)&ADC_MASK];
}

void C_ADC::medianGlattung()
{
    //MEDIAN
    int arylen = sizeof(median)/sizeof(median[0]);
    uint16_t sAry[arylen];


    median[2] = sAry[2] = median[1];
    median[1] = sAry[1] = median[0];
    median[0] = sAry[0] = adcAry[adcAryIndex];

    for(int i=0; i< arylen -1 ; ++i)
        for(int j=0 ; j < arylen -i-1 ; ++j)
            if(sAry[j] > sAry[j+1])
            {
                uint16_t tmp = sAry[j];
                sAry[j] = sAry[j+1];
                sAry[j+1] = tmp;
            }

    /******Medianwert ins Array uebernehmen************/
    adcAry[adcAryIndex] = median[arylen/2] = sAry[arylen/2];
    return;
}

void C_ADC::slidingAverage(void)
{
    //Sliding Average
    uint64_t mw = 0;
    int arylen = sizeof(slAverage) / sizeof(slAverage[0]);

    //Medianwert hinten in das Schieberegister einfuegen
    slAverage[arylen-1] = adcAry[adcAryIndex];

    //Mittelwert bilden
    for(int i = 0 ; i < arylen ; ++i)
        mw += slAverage[i];
    mw /=arylen;

    //Mittelwert hinten an das Schieberegister einfuegen
    slAverage[arylen-1] = static_cast<uint16_t>(mw);

    //Schieberegister takten
    for(int i = 0 ; i < (arylen-1)  ; ++i)
        slAverage[i] = slAverage[i+1];

    //in das AuswerteArray eintragen
    adcAry[adcAryIndex] = static_cast<uint16_t>(mw);

    return;
}

/*
double C_ADC::COR(void)
{
//empirischen Korrelationskoeffizient berechnen

    long tmparyCOR[MAX_COR] = {0};

     long long xSquareSum = 0 ; long ySquareSum =0;
     long long XiYiSum = 0;
     long double XiQuer = 0.0 ; long double YiQuer = 0.0;
     long long XiSum = 0 ; long long YiSum =0;
     long double xNenner = 0.0 ; long double yNenner = 0.0 ; long double zaehler = 0.0;

    for(int i=0 ; i<MAX_COR ; ++i)
    {

        tmparyCOR[i] = aryCOR[i] - COR_OFFSET +1*i; //+i fuer Steigung
        xSquareSum += (i*i);
        ySquareSum += tmparyCOR[i] * tmparyCOR[i];
        XiYiSum += i * tmparyCOR[i];
        XiSum += i;
        YiSum += tmparyCOR[i];

    }

    XiQuer = (double)XiSum / (double)MAX_COR;
    YiQuer = (double)YiSum / (double)MAX_COR;

    zaehler = (double)XiYiSum - (double)MAX_COR * XiQuer * YiQuer;
    xNenner = (double)xSquareSum - (double)MAX_COR * (XiQuer*XiQuer) ;
    yNenner = (double)ySquareSum - (double)MAX_COR * (YiQuer*YiQuer);


    return corVal = fabs( (zaehler / sqrt( xNenner * yNenner))  );
}
*/

uint16_t C_ADC::getValue()
{
    return this->value;
}


void C_ADC::reset()
{
   ;
}

bool C_ADC::averageValid(void)
{
    return this->averValid;
}

uint16_t C_ADC::averageValue(void)
{
    averNiveau = 0;
    if( adcAry[1799] == 0)
    {
        averValid = false;
        return -1;
    }
    else
    {
        long long tmp = 0;
        for(int i = 0 ; i < 1800 ; ++i)
        {
            tmp += adcAry[i];
        }

        tmp = tmp / 1800;
        averNiveau = static_cast<uint16_t>(tmp);
        averValid = true;

        return averNiveau;
    }
}

uint16_t C_ADC::getAverageNiveau(void)
{
    return this->averNiveau;
}

C_ADC::~C_ADC()
{
    ;
}

